﻿Type=Class
Version=2.7
@EndOfDesignText@
'Class module
'
' Forked from V1.5 of HotSpot Code Module
'
'Written by Steve Laming
'
' Options:
' Put the name of the Layout that defines where the hotspots should be in the customview object's tag in the designer e.g. HotSpot.bal.
' The layout should contain a Panel named Panel1 which then contains:
' Panels For hotspot rectangles and
' Labels for hot circles (No elipses I'm afraid)
'
' If Panel1's tag contains copy, then the background image will be copied to the Base panels image.
'
Sub Class_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
	
	Private HotSpotsList As List
	Type HotSpotType(ID As Int,HSXIndex As Int,HSYIndex As Int,Rectangle As Rect,Valid As Boolean,Tag As Object,Type As String)
	Dim Initialized As Boolean = False
	Private Panel1 As Panel
	Private cEventName As String
	Private cModule As Object
	Private cBase As Panel
	Private cLbl As Label

End Sub
'If Using the designer DO NOT Call this sub.
'Otherwise pass (Me,"EventName")
'In either case set up a sub called {EventName}_Touch(Param() as Object) in your calling module.
'Variables need to be pulled from the array As:
'	Dim Action As Int = Param(0)
'	Dim X As Float = Param(1)
'	Dim Y As Float = Param(2)
'
Public Sub Initialize (TargetModule As Object, EventName As String)
    cModule = TargetModule
    cEventName = EventName
	HotSpotsList.Initialize
	Initialized = True
	
End Sub
'Used by the designer, DO NOT USE for manually created HotSpotClass
Public Sub DesignerCreateView(Base As Panel, Lbl As Label, Props As Map)

	cBase = Base
	cLbl=Lbl
	
	'Add a touch listener to Base panel
	Dim R As Reflector
	R.Target=cBase
	R.SetOnTouchListener("Base_Touch")
	
	'Check if a file is specified to create Hotspots from
	If cLbl.Text <> "" Then
		'Can't load a layout in this sub, probably because it is still creating the initial layout.
		'Use CallsubDelayed to do it when finished.
		CallSubDelayed3(Me,"PopulateFromLayout",cLbl.Text,False)
	End If
	
End Sub
'Called delayed from DesignerCreateView populates the HotSpots based on the contents of Panel1 in the layout name stored in
'Lbl text passed from the designer.
'You can also manually populate from a Layout, same rules apply.  Pass the filename of a layout .bal 
' The layout should contain a Panel named Panel1 which then contains:
' Panels For hotspot rectangles and
' Labels for hot circles
'Copy Is OR'ed so either 'copy' in the Panel1.Tag or True passed will cause a copy.
Sub PopulateFromLayout(FileName As String, Copy As Boolean) As Int

	'Check valid layoutfile name and load
	If Not(FileName.ToLowerCase.EndsWith(".bal")) Then FileName = FileName&".bal"
	If File.Exists(File.DirAssets,FileName) Then
		cBase.LoadLayout(FileName)		'Load the layout
	Else
		'Invalid file
		Log("HotSpotClass: Info - No layout name suplied in Lbl.Text not creating HotSpots, or layout file "&FileName&" does not exist")
		Return -1
	End If

	'Create HotSpots Rect's for Panels and Circles for Labels
	For i = 0 To Panel1.NumberOfViews-1
		Dim v As View = Panel1.GetView(i)
		If v Is Panel Then
				AddHotSpot(v.Left,v.Top,v.Left+v.Width,v.Top+v.Height,v.Tag)
		Else
			If v Is Label Then
				AddHotCircle(v.Left,v.Top,v.Width,v.Tag)
			End If
		End If
	Next
	
	'If Panel1's tag contains 'copy', then copy it's background image to Base
	Dim P1Tag As String = Panel1.Tag
	If P1Tag.ToLowerCase ="copy" OR Copy Then 
		cBase.Background=Panel1.Background
	Else
		Log("HotSpotClass: Info - Copy background image from Panel1 ('copy' in tag) not specified")
	End If
	
	'Finished with Panel1 we can remove it.
	Panel1.RemoveView
	Return 0
End Sub
' For manual code use, do not call if defined in the designer.
' Pass the Activity AND parameters that define the Panel 
' on which touch will be monitored For the hotspots.
Sub SetUp(Module As Object,Left As Int,Top As Int,Width As Int,Height As Int)
	Panel1.Initialize("")
	Dim R As Reflector
	R.Target=Panel1
	R.SetOnTouchListener("Base_Touch")
	If Module Is Panel Then
		Dim tModule As Panel = Module
		tModule.AddView(Panel1,Left,Top,Width,Height)
	Else
		If Module Is Activity Then
			Dim tAct As Activity = Module
			tAct.AddView(Panel1,Left,Top,Width,Height)
		End If
	End If
End Sub
'Callback for panel touch, redirect to Calling Activity sub
Sub Base_Touch (ViewTag As Object,Action As Int, X As Float, Y As Float,MotionEvent As Object) As Boolean
	Dim Param(3) As Object
	Param(0)=Action
	Param(1) = X
	Param(2) = Y
	CallSub2(cModule,cEventName&"_Touch",Param)
	Return True
End Sub 
'Create a hotspot
'HotSpotLib.AddHotSpot(100,150,130,180,"HS 1") will create a hotspot
'at X=100, Y=150 with a width of 30 and a height of 30
Sub AddHotSpot(cLeft As Int,cTop As Int, cRight As Int,cBottom As Int,cTag As Object) As Int

	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return -1
	End If
	
	Dim R As Rect
	R.Initialize(cLeft,cTop,cRight,cBottom)
	
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=cLeft
	H.HSYIndex=cTop
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type ="S"
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'Create a hotspot from a rect
'HotSpotLib.AddHotSpot(Rect,"HS 1") will create a hotspot
'with the dimensions of the rectangle passed
Sub AddHotSpotRect(R As Rect,cTag As Object) As Int

	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return
	End If
	
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=R.Left
	H.HSYIndex=R.Top
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type ="S"
	Log(H)
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'HotSpotLib.AddHotCircle(20Dip,20Dip,80Dip,"Tag4") will create a hotspot at
'Left 20dip,Top 20Dip, Diameter 80dip. Center = 20+(80/2) = 60.
Sub AddHotCircle(cLeft As Int,cTop As Int, cDiameter As Int,cTag As Object) As Int
	If Not(Initialized) Then
		Msgbox("HotSpot module must be initialized...","Initialize Error")
		Log("HotSpot module must be initialized...")
		Return
	End If
	Dim R As Rect
	R.Initialize(cLeft,cTop,cLeft+cDiameter,cTop+cDiameter)
	Dim H As HotSpotType
	H.Initialize
	H.HSXIndex=cLeft
	H.HSYIndex=cTop
	H.Rectangle=R
	H.Valid=True
	H.Tag=cTag
	H.Type="C"
	H.ID=HotSpotsList.Size
	HotSpotsList.Add(H)
	Return H.ID
End Sub
'Turns off hotspot, returns the ID if successful or -1 if it doesnt exist
Sub DeactivateHotSpot(ID As Int) As Int
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Valid=False
		Return ID
	Else
		Return Pos
	End If
End Sub
'Edit an existing HotSpot
Sub EditHotSpot(mID As Int,mLeft As Int,mTop As Int, mRight As Int,mBottom As Int,mTag As Object) As Int
	
	Dim Pos As Int
	Pos=FindIndex(mID)
	If Pos <> -1 Then
		Dim R As Rect
		R.Initialize(mLeft,mTop,mRight,mBottom)
		Dim H As HotSpotType
		H=HotSpotsList.get(Pos)
		
		H.Initialize
		H.HSXIndex=mLeft
		H.HSYIndex=mTop
		H.Rectangle=R
		H.Valid=True
		H.Tag=mTag
		H.ID=mID
		HotSpotsList.RemoveAt(Pos)
		HotSpotsList.Add(H)
		Return H.ID
	End If
End Sub
'Turns on hotspot, returns the ID if successful or -1 if it doesnt exist
Sub ReActivateHotspot(ID As Int) As Int
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Valid=True
		Return ID
	Else
		Return Pos
	End If
End Sub
'Change HotSpotType 'S'quare or 'C'ircle
Sub ChangeType(ID As Int,mType As String) As Int
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		HS.Type=mType
		Return ID
	Else
		Return Pos
	End If
End Sub
'Readable output of object from ID
Sub ToStringID(ID As Int) As String
	Dim Pos As Int
	Pos=FindIndex(ID)
	If Pos <> -1 Then
		Dim HS As HotSpotType = HotSpotsList.get(Pos)
		Return ("ID "&HS.ID&" X "&HS.HSXIndex&" Y "&HS.HSYIndex&" Rect "&HS.Rectangle&" Valid "&HS.Valid&" Tag "&HS.Tag)
	Else
		Return Pos
	End If
End Sub
'Readable output of object from Object
Sub ToStringHS(HS As HotSpotType) As String
	Return ("ID "&HS.ID&" X "&HS.HSXIndex&" Y "&HS.HSYIndex&" Rect "&HS.Rectangle&" Valid "&HS.Valid&" Tag "&HS.Tag)
End Sub
'Find the HotSpot from the ID via Binary search

Private Sub FindIndex(Val As Int) As Int
	Dim iMax,iMin,iMid As Int
	HotSpotsList.SortType("ID",True)
	iMax=HotSpotsList.Size-1
	iMin=0
	
	'Set up for first iteration
	iMid=iMin+((iMax-iMin)/2)
	Dim HS As HotSpotType = HotSpotsList.get(iMid)
	
	'Start
	Do While iMin < iMax
		'1st or second half
		If HS.ID < Val Then
			iMin=iMid+1
		Else
			iMax=iMid
		End If
		'Set pointer to half way for subsequent iterations
		iMid=iMin+((iMax-iMin)/2)
		Dim HS As HotSpotType = HotSpotsList.get(iMid)
	Loop
	
	'Check if found
	If iMax=iMin AND HS.ID=Val Then
		Return iMin
	Else
		Return -1
		End If
End Sub
'Finds a hot spot by searching the x axis then the y axis
Sub FindHotSpot(X As Float, Y As Float) As HotSpotType

	'Sort in required order Y within X
	HotSpotsList.SortType("HSYIndex",True)
	HotSpotsList.SortType("HSXIndex",True)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	Dim I As Int = 0
	Do While I < HotSpotsList.Size
		'Get the first hotspot to check
		Dim HS As HotSpotType = HotSpotsList.get(I)

		'Not Found if we have passed the required position
		If HS.Rectangle.Left > X Then Return NotFound
		
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X Then
			'X is correct
			Dim j As Int = 0
		
			'Check Y Axis - Make sure we  are still within the X Axis and haven't run out of hotspots
			Do While HS.Rectangle.Left <= X AND I+j < HotSpotsList.Size
				'Reading the first one again, but didn't want to change logic just yet.
				Dim HS As HotSpotType = HotSpotsList.get(I+j)
			
				'Already passed it
				If HS.Rectangle.Top > Y Then Return NotFound
				
				'Fit's the Y Axis
				If HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then
				
					'Have we deactivated this one?
					If HS.Valid Then
						'Not A Circle
						If HS.Type = "S" Then Return HS
						'Addl Check
						If InCircle(HS.Rectangle,X,Y) Then Return HS
						Return NotFound
					Else
						Return NotFound
					End If
				End If
				j=j+1
			Loop
			
			'Finished checking subsequent Y Axis and should now have finished the search but send it back anyway
			I=I+j
		End If
		I=I+1
	Loop
	
	'Found nothing
	Return NotFound
End Sub
'Finds all hot spots by looking at each one, returns a list of found hotspots or a list with an uninitialised hotspot
Sub FindHotSpotAll(X As Float, Y As Float) As List

	Dim HSList As List
	HSList.Initialize
	Dim NotfoundList As List
	NotfoundList.Initialize

	'Sort not required as will check all
		
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then HSList.Add(HS)
	Next
		
	If HSList.Size > 0 Then Return HSList
	'Found nothing
	Return NotfoundList
End Sub
'Find a hotspot by looking through a list in descending order of creation, will return the last hotspot created that is touched.
Sub FindHotSpotLast(X As Float, Y As Float) As HotSpotType

	'Sort in required reverse creation order
	HotSpotsList.SortType("ID",False)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then Return HS
	Next
	Return NotFound
End Sub
'Find a hotspot by looking through a list in order of creation, will return the first hotspot created that is touched.
Sub FindHotSpotFirst(X As Float, Y As Float) As HotSpotType

	'Sort in required reverse creation order
	HotSpotsList.SortType("ID",True)
	
	'Create a dummy object to return if not found
	Dim NotFound As HotSpotType
	
	'Start looking in X Axis
	For Each HS As HotSpotType In HotSpotsList
		'Found HotSpot on the X Axis
		If HS.Rectangle.Left <= X AND HS.Rectangle.Right >= X AND HS.Rectangle.Top <= Y AND HS.Rectangle.Bottom >= Y Then Return HS
	Next
	Return NotFound
End Sub

Private Sub InCircle(R As Rect,X As Float,Y As Float) As Boolean

	Dim Distance As Float
	Distance = Sqrt(Power(X-R.CenterX,2)+Power(Y-R.CenterY,2))
	If Distance > (R.Right-R.Left)/2 Then Return False
	Return True
End Sub
''Returns the current initialized state of the lib.  To initialize, add a hotspot.
'Sub IsInitialized As Boolean
'	Return HotSpotsList.IsInitialized
'End Sub
''Returns a rect o the hotspot
Sub GetRectangle(ID As Int) As Rect
	Dim HS As HotSpotType = HotSpotsList.Get(FindIndex(ID))
	Return HS.Rectangle
End Sub
'Clear all hotspots from the list
Sub ClearHotSpots
	HotSpotsList.Clear
End Sub
'Returns the current initialized state of the lib.
Sub IsInitialized As Boolean
	Return Initialized
End Sub
Sub GetPanel As Panel
	Return Panel1
End Sub